<?php
/*
Plugin Name: Biuro HTML
Description: Biuro Biuro HTML plugin
Author: Biuro
Version: 1.0
Author URI: https://www.biuro.lt/
*/


class Biuro_HTML extends WP_Widget {
    // Main constructor
    public function __construct() {
        parent::__construct(
            'biuro-html',
            __( 'Biuro HTML | Biuro', 'biuro' ),
            array(
                'customize_selective_refresh' => true,
            )
        );
    }

    // The widget form (for the backend )
    public function form( $instance ) {
        // Set widget defaults
        $defaults = array(
            'title' => '',
            'content' => '',
        );

        // Parse current settings with defaults
        extract( wp_parse_args( ( array ) $instance, $defaults ) ); ?>

        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php _e( 'Title', 'biuro' ); ?><?php echo ' ' . $i . ':';  ?></label>
            <input class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text" value="<?php echo esc_attr( ${'title'} ); ?>" />
        </p>
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>"><?php _e( 'Content:', 'biuro' ); ?></label>
            <textarea class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'content' ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( 'content' ) ); ?>" rows="12"><?php echo $content; ?></textarea>
        </p>

    <?php }

    // Update widget settings
    public function update( $new_instance, $old_instance ) {
        $instance = $old_instance;
        $instance['title']    = isset( $new_instance['title'] ) ? wp_strip_all_tags( $new_instance['title'] ) : '';
        $instance['content'] = isset( $new_instance['content'] ) ? $new_instance['content'] : '';
        return $instance;
    }

    // Display the widget
    public function widget( $args, $instance ) {
        extract( $args );

        // Check the widget options
        $title = isset( $instance['title'] ) ?$instance['title'] : '';
        $content = isset( $instance['content'] ) ?$instance['content'] : '';

        // WordPress core before_widget hook (always include )
        echo $before_widget;

        if ( $title != "" ) {
            echo '<h3>' . $title . '</h3>';
        }

        if ( $content != "" ) {
            // echo '<p>' . $content . '</p>';
            echo $content;
        }

        // WordPress core after_widget hook (always include )
        echo $after_widget;
    }
}


// register Biuro_HTML
add_action( 'widgets_init', function(){
    register_widget( 'Biuro_HTML' );
});

