// https://github.com/webpack/webpack/issues/8656#issuecomment-456001556

// import "core-js/es";

import './polyfills/polyfills';

import { inititateRecaptcha } from './core/';

import { getAccordion, getFeedbackSwiper, getAutocomplete, getMaps, getJobShare } from './components/components';

if (module.hot) {
    module.hot.accept();
}

(global => {
    const container = document.getElementById('cookie-warning');
    const btnAgree = document.getElementById('cookie-agree');
    const btnClose = document.getElementById('cookie-close');
    const agreed = (useLS()) ? localStorage.getItem('biuro-agree') : false;

    function useLS () {
        var mod = 'a';
        try {
            localStorage.setItem(mod, mod);
            localStorage.removeItem(mod);
            return true;
        } catch (e) {
            return false;
        }
    }

    if (!container || !btnAgree || !btnClose || agreed) {
        return;
    }

    container.style.display = 'block';

    btnAgree.addEventListener('click', () => {
        if (useLS()) {
            localStorage.setItem('biuro-agree', 'true');
        }
        container.style.display = 'none';
    });

    btnClose.addEventListener('click', () => {
        container.style.display = 'none';
    });
})(window);

(global => {
    const node = document.querySelector('.c-nav--lang');

    if (!node) {
        return;
    }

    node.addEventListener('click', (e) => {
        e.currentTarget.classList.toggle('c-nav--lang--is-open');
    });
})(window);

(global => {
    document.querySelectorAll('.js-toggle--nav').forEach((node) => {
        node.addEventListener('click', (e) => {
            e.preventDefault();
            document.body.classList.toggle('is-nav--open');
        });
    });

    document.querySelectorAll('.js-toggle--aside').forEach((node) => {
        node.addEventListener('click', (e) => {
            e.preventDefault();
            document.body.classList.remove('is-aside--open');
            document.body.classList.remove('is-aside--open-additional');
            document.body.classList.remove('is-aside--open-city');
        });
    });

    document.querySelectorAll('.js-toggle--aside-city').forEach((node) => {
        node.addEventListener('click', (e) => {
            e.preventDefault();
            document.body.classList.add('is-aside--open');
            document.body.classList.remove('is-aside--open-additional');
            document.body.classList.add('is-aside--open-city');
        });
    });

    document.querySelectorAll('.js-toggle--aside--additional').forEach((node) => {
        node.addEventListener('click', (e) => {
            e.preventDefault();
            document.body.classList.toggle('is-aside--open');
            document.body.classList.remove('is-aside--open-city');
            document.body.classList.add('is-aside--open-additional');
        });
    });
})(window);

document.querySelectorAll('.js-form--input-file').forEach((node) => {
    node.addEventListener('change', () => {
        const parent = node.parentNode;
        const name = node.value.split(/\\|\//).pop();
        const text = parent.querySelector('.js-form--input-file-text');

        if (text && name) {
            text.innerHTML = name;
        }
    });
});

document.querySelectorAll('.js-toggle--services').forEach((node) => {
    node.addEventListener('click', () => {
        node.parentNode.classList.toggle('c-services--is-open');
    });
});

document.querySelectorAll('.js-toggle--sections').forEach((node) => {
    node.addEventListener('click', () => {
        const parent = node.parentNode;

        if (parent.classList.contains('c-sections--is-open')) {
            parent.classList.toggle('c-sections--is-open');
        } else {
            document.querySelectorAll('.c-sections--is-open').forEach((node) => {
                node.classList.remove('c-sections--is-open');
            });
            parent.classList.toggle('c-sections--is-open');
        }
    });
});

document.querySelectorAll('.js-expand-jobs-section').forEach((node) => {
    node.addEventListener('click', (e) => {
        e.preventDefault();
        node.parentNode.classList.remove('c-jobs-section--is-closed');
        node.remove();
    });
});

const initiatePositionForm = () => {
    const button = document.querySelector('[name="s"][value="1"]');

    if (button) {
        button.click();
        return;
    }

    const input = document.querySelector('[name="name"]');

    if (input) {
        input.focus();
    }
};

if (document.querySelector('.js-inititate-position-form')) {
    document.querySelector('.js-inititate-position-form').addEventListener('click', initiatePositionForm);
}

if (document.querySelector('.js-focus-form')) {
    document.querySelector('.js-focus-form').addEventListener('click', () => {
        const input = document.querySelector('.c-form input:first-of-type');

        if (input) {
            input.focus();
        }
    });
}

if (document.querySelector('.js-modal--close')) {
    document.querySelector('.js-modal--close').addEventListener('click', () => {
        const modal = document.querySelector('.c-modal');

        if (modal) {
            modal.remove();
        }

        const node = document.querySelector('.c-form [autofocus]');

        if (node) {
            node.focus();
        }
    });
}

(global => {
    const node = document.querySelector('.c-form [autofocus]');

    if (node) {
        node.focus();
    }

    if (location.hash) {
        const error = document.querySelector('.c-form--input-wrap--error input') || document.querySelector('.c-form--checkbox--error');

        if (error) {
            error.focus();
            return;
        }

        const input = document.querySelector('.c-form [name="name"]');

        if (input) {
            input.focus();
        }
    }
})(window);

(global => {
    document.querySelectorAll('.gtm-share-click').forEach((node) => {
        node.addEventListener('click', (e) => {
            const dataset = node.dataset || {};
            window.dataLayer = window.dataLayer || [];
            window.dataLayer.push({
                event: 'shareClick',
                shareLabel: dataset.label || ''
            });
        });
    });

    document.querySelectorAll('.gtm-click').forEach((node) => {
        node.addEventListener('click', (e) => {
            const dataset = node.dataset || {};
            window.dataLayer = window.dataLayer || [];
            window.dataLayer.push({
                event: dataset.event || '',
                label: dataset.label || ''
            });
        });
    });

})(window);

const submitContactsFrom = (form) => {
    if (form.querySelector('[type="submit"]')) {
        form.querySelector('[type="submit"]').classList.add('c-btn--disabled');
        form.querySelector('[type="submit"]').classList.add('c-btn--loading');
    }
};

document.querySelectorAll('[name="form-position"], [name="form-employees"], [name="form-employers"], [name="form-recommend"]').forEach((form) => {
    form.addEventListener('submit', () => { submitContactsFrom(form); });
});

if (document.querySelector('#submit-recommend')) {
    document.querySelector('#submit-recommend').addEventListener('click', (e) => {
        e.preventDefault();

        const button = document.querySelector('[name="form-recommend"] [type="submit"]');

        if (!button) {
            return;
        }

        e.currentTarget.classList.add('c-btn--disabled');
        e.currentTarget.classList.add('c-btn--loading');

        button.click();
    });
}

if (document.querySelector('.js-submit-employees-form')) {
    document.querySelector('.js-submit-employees-form').addEventListener('click', (e) => {
        const button = document.querySelector('[name="form-employees"] [type="submit"]');

        if (!button) {
            return;
        }

        e.preventDefault();

        e.currentTarget.classList.add('c-btn--disabled');
        e.currentTarget.classList.add('c-btn--loading');

        button.click();
    });
}

if (document.querySelector('.js-accordion--header')) {
    getAccordion().then(inititateAccordion => {
        inititateAccordion();
    });
}

const inititateFeedbacks = (node) => {
    if (!node) {
        return;
    }

    const inititate = () => {
        getFeedbackSwiper().then(inititateFeedbackSwiper => {
            inititateFeedbackSwiper();
        });
    };

    if (!window.IntersectionObserver) {
        inititate();
        return;
    }

    const io = new IntersectionObserver((entries) => {
        entries.forEach(entry => {
            if (entry.isIntersecting) {
                inititate();
                io.unobserve(entry.target);
            }
        });
    }, {});

    io.observe(node);
};

inititateFeedbacks(document.querySelector('.c-feedbacks--inner'));

const inititateAutocompletes = () => {
    const inititate = () => {
        getAutocomplete().then(inititateAutocomplete => {
            inititateAutocomplete();
        });
    };

    if (window.requestIdleCallback) {
        window.requestIdleCallback(inititate);
    } else {
        inititate();
    }
};

if (document.getElementById('search') || document.getElementById('form-city') || document.getElementById('recommend-form-city')) {
    inititateAutocompletes();
}

if (document.getElementById('js-map--divisions') || document.getElementById('js-map--cities') || document.getElementById('js-map--regions')) {
    getMaps().then(inititateMap => {
        inititateMap();
    });
}

if (document.querySelector('.js-copy-to-clipboard') || document.querySelector('.js-share-messenger') || document.querySelector('.js-share-whatsapp') || document.querySelector('.js-biuro-facebook')) {
    getJobShare().then(inititateJobShare => {
        inititateJobShare();
    });
}

if (document.querySelector('.js-collapse-row--toggle')) {
    document.querySelector('.js-collapse-row--toggle').addEventListener('click', (e) => {
        e.currentTarget.parentNode.classList.toggle('c-form--row--collapsed');
    });
}

const showPopup = (e) => {
    window.dataLayer = window.dataLayer || [];

    if (e.clientY < 50 && e.relatedTarget == null && e.target.nodeName.toLowerCase() !== 'select') {
        document.removeEventListener('mouseout', showPopup);

        if (window.sessionStorage) {
            window.sessionStorage.setItem('popup-closed', 'true');
        }

        if (document.getElementById('popup')) {
            document.getElementById('popup').style.display = 'block';
            window.dataLayer.push({ event: 'b2c_popup_show' });
        }

        if (document.querySelector('.js-popup--btn')) {
            document.querySelector('.js-popup--btn').addEventListener('click', (e) => {
                window.dataLayer.push({ event: 'b2c_popup_navigate', label: 'Recommend page' });
            });
        }

        if (document.querySelector('.js-popup--close')) {
            document.querySelector('.js-popup--close').addEventListener('click', (e) => {
                e.preventDefault();

                if (document.getElementById('popup')) {
                    document.getElementById('popup').style.display = 'none';
                }
            });
        }
    }
};

if (document.getElementById('popup') && (!window.sessionStorage || !window.sessionStorage.getItem('popup-closed'))) {
    setTimeout(() => {
        document.addEventListener('mouseout', showPopup);
    }, 3000);
}

const recaptcha = document.getElementById('g-recaptcha-response');

if (recaptcha) {
    inititateRecaptcha(recaptcha);
}
