<?php
/**
 * Template part for displaying a list of jobs
 *
 * @link https://developer.wordpress.org/themes/basics/template-hierarchy/
 *
 * @package WordPress
 * @subpackage Biuro
 * @since 1.0.0
 */

global $jobs;
$date_format = get_option( 'date_format' );

function showPosted($published, $valid) {
    $today = date('Y-m-d');

    if ($published == $today) :
        _e('Today', 'biuro');
    else:
        $now = new DateTime($today);
        $till = new DateTime($valid);

        $count = $till->diff($now)->format("%a") + 1;

        if ($count < 4):
            printf( _n( '%d day left!', '%d days left!', $count, 'biuro'  ), $count );
        else:
            $count = $now->diff(new DateTime($published))->format("%a");

            printf( _n( '%d day ago', '%d days ago', $count, 'biuro'  ), $count );
        endif;
    endif;
}

?>

    <table class="c-jobs--table">
    <?php
        while ( $jobs->fetch() ) :
            $ID =  $jobs->display( 'ID' );
            $name = $jobs->display( 'name' );
?>
        <tr>
            <td class="c-jobs--col c-jobs--col-position">
                <a class="c-jobs--anchor" href="<?php echo get_post_permalink( $ID ); ?>" title="<?php echo $name; ?>"><?php echo $name; ?></a>
            </td>
            <td class="c-jobs--col c-jobs--col-city">
<?php
                $cities = $jobs->field( 'city');
                $count = 0;

                //Primary city id
                $primaryCities = get_post_meta($ID, '_yoast_wpseo_primary_city');

                //Sort by primary city (put it first)
                if (is_array($cities)):
                    uasort($cities, function($a, $b) use ($primaryCities) {
                        if (count($primaryCities) == 0) :
                            return 0;
                        endif;

                        if ($a['term_id'] == $primaryCities[0]) :
                            return -1;
                        endif;

                        return 1;
                    });

                    foreach($cities as $city):
                        if ($count):
                            echo  ', ';
                        endif;

                        echo $city['name'];

                        $count++;
                    endforeach;
                else:
?>              &nbsp;
<?php
                endif;
?>
            </td>
            <td class="c-jobs--col c-jobs--col-valid">
                <?php showPosted( substr($jobs->field('date'), 0, 10), $jobs->field( 'valid' )); ?>
            </td>
        </tr>

<?php
        endwhile;
?>
    </table>

<?php

    function getPaginationURL($i) {
        $pathname = preg_replace('/[?|^&]pg=\\d+/', '', $_SERVER['REQUEST_URI']);

        if ($i === 1) {
            return $pathname;
        }

        $append = '?';

        if ( false !== strpos( $pathname, '?' ) ) {
            $append = '&';
        }


        return $pathname . $append . 'pg=' . $i;
    }

    function getPaginationPage($i, $page) {
        if ($i == $page):
            echo '<span aria-current="page" class="page-numbers current">' . $i . '</span>';
        else:
            echo '<a class="page-numbers" href="' . getPaginationURL($i) . '">' . $i . '</a>';
        endif;
    }

    if ( !is_front_page() ):

        $total = ceil($jobs->total_found()/$jobs->limit);
        $page = $jobs->page;
        $prev = ($total > 7 && $page > 3) ? true : false;
        $next = ($total > 7 && $page < ($total - 3) ) ? true : false;

        if ($total > 1) :
            echo '<span class="pods-pagination-paginate ">';

            if ($page > 1):
                echo '<a class="prev page-numbers" href="' . getPaginationURL($page-1) . '"> </a>';
            endif;

            getPaginationPage(1, $page);

            if ($prev):
                echo '<a class="page-numbers dots" href="' . getPaginationURL($page-3) . '">...</a>';
            endif;

            if ($total < 8):
                for ($i = 2; $i < $total; $i++):
                    getPaginationPage($i, $page);
                endfor;
            else:
                if ($page < 4):
                    for ($i = 2; $i <= 5; $i++):
                        getPaginationPage($i, $page);
                    endfor;
                elseif ($page > ($total - 4)):
                    for ($i = $total - 4; $i < $total; $i++):
                        getPaginationPage($i, $page);
                    endfor;
                else:
                    for ($i = $page - 1; $i <= $page + 1; $i++):
                        getPaginationPage($i, $page);
                    endfor;
                endif;
            endif;

            if ($next):
                echo '<a class="page-numbers dots" href="' . getPaginationURL($page+3) . '">...</a>';
            endif;

            getPaginationPage($total, $page);

            if ($page < $total):
                echo '<a class="next page-numbers" href="' . getPaginationURL($page+1) . '"> </a>';
            endif;
            echo '</span>';
        endif;

        // echo $jobs->pagination( array(
        //     'type' => 'paginate' ,
        //     'prev_text' => ' ',
        //     'next_text' => ' '
        // ) );
    endif;

?>
