<?php

function getPageURL($taxonomy, $pageURL, $searchQuery, $termID) {
    $term = get_term($termID);

    if (!$term || !$term->name):
        return $pageURL;
    endif;

    if (isset($searchQuery['s']) && $searchQuery['s'] && $taxonomy == 'city'):
        return '/?city=' . $term->name . '&s=' . $searchQuery['s'];
    elseif (isset($searchQuery['city']) && $searchQuery['city'] && ($taxonomy == 'type' || $taxonomy == 'field')):
        return '/?city=' . $searchQuery['city'] . '&s=' . $term->name;
    endif;

    return $pageURL;
}

function getSiteTree($type, $section, $searchQuery, $term) {
    $res = array();
    $termID = $term != '' ? $term->term_id : '';

    $pages = getOptions($type);

    foreach ( $pages as $page ):
        if ($page['pageID'] && $page['status'] == 'publish') :
            array_push($res, array(
                'name' => $page['title'],
                'href' => getPageURL($type, get_page_link( $page['pageID'] ), $searchQuery, $page['termID']),
                'total' => $page['total'],
                'active' => $page['termID'] == $termID ? true : false
            ));
        endif;
    endforeach;

    $counter = count($res);

    if ($counter == 0) {
        return '';
    }

    $classCSS = 'c-jobs-section';

    $isClosed = $counter > 3;

    if ($isClosed):
        $classCSS .= ' c-jobs-section--is-closed';
    endif;

    $classCSS .= ($type == 'city') ? ' c-jobs-section--city' : ' c-jobs-section--additional';

    echo '<div class="' . $classCSS . '">';
        echo '<h3 class="c-jobs-section--heading">';
            echo '<svg width="16px" height="19px" class="c-ico--' . $type . '--job-section">';
                echo '<use href="#ico--' . $type . '"></use>';
            echo '</svg>';

            echo $section;
        echo '</h3>';

        echo '<div class="c-jobs-section--content">';
            echo '<ul  class="c-jobs-section--list">';

            foreach ( $res as $r ):
?>
                <li class="c-jobs-section--item <?php if ($r['active']): echo 'c-jobs-section--item-active'; endif; ?>"><a href="<?php echo $r['href']; ?>" title="<?php echo $r['name']; ?>" class="c-jobs-section--anchor"><?php echo $r['name'] . ' (' . $r['total'] . ')'; ?></a></li>
<?php
            endforeach;

         echo '</ul>';

        echo '</div>';

    if ($isClosed):
        echo '<a href="#" class="js-expand-jobs-section c-jobs-section--expand">' . __('More ...', 'biuro') . ' </a>';
    endif;

    echo '</div>';
}

    global $query_string;

    wp_parse_str( $query_string, $searchQuery );

    $pageTerms = getPageTerm(get_post_custom_keys());

    if ($pageTerms):
        if ($pageTerms->taxonomy === 'type' || $pageTerms->taxonomy === 'field'):
            $searchQuery['s'] = $pageTerms->name;
        elseif ($pageTerms->taxonomy === 'city'):
            $searchQuery['city'] = $pageTerms->name;
        endif;
    endif;

    $city = '';
    $field = '';
    $type = '';

    if (isset($searchQuery['city'])):
        $city = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['city']), 'city' );
    endif;

    if (isset($searchQuery['s'])):
        $field = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['s']), 'field' );
        $type = get_term_by( 'name', preg_replace('/[\+]/', ' ', $searchQuery['s']), 'type' );
    endif;

    getSiteTree( 'city', __('City', 'biuro'), $searchQuery, $city );
?>

<?php
    getSiteTree( 'field', __('Job area', 'biuro'), $searchQuery, $field );
?>

<?php
    getSiteTree( 'type', __('Job type', 'biuro'), $searchQuery, $type );
?>

<?php
    global $positionsPage;

    if ($positionsPage):
?>
    <p class="c-search--reset">
        <a href="<?php echo $positionsPage; ?>" class="o-btn c-btn--main"><?php _e('Reset filters', 'biuro'); ?></a>
    </p>
<?php
    endif;
?>

<?php

$VSD = get_term_by( 'slug', 'type-8-lt', 'type' );

if ($VSD && isset($searchQuery['s']) && $VSD->name == $searchQuery['s']): ?>

    <div class="c-vdb--section c-vdb--section-aside">
        <img src="/wp-content/themes/biuro/i/vdb/not-found.png" width="85px" height="85px" alt="VDB not found" class="c-vdb--section-img" />
        <h3 class="c-vdb--section-heading"><?php _e("Didn't find a job offer you like?", 'biuro'); ?></h3>
        <p class="c-vdb--section-content">
            <?php _e('Fill out the form and we will contact you', 'biuro'); ?>
        </p>
    <?php
        $contactsPage = new WP_Query( array(
            'numberposts'   => 1,
            'post_type'     => 'page',
            'meta_key'      => 'page',
            'meta_value'    => 'employees-contacts'
        ) );

        if ( $contactsPage->have_posts() ) :
            while( $contactsPage->have_posts() ) :
                $contactsPage->the_post();
        ?>
                <a href="<?php echo get_the_permalink(); ?>?#action" class="o-btn c-btn--vdb"><?php _e('Get a job offer', 'biuro'); ?></a>
        <?php
            endwhile;
        endif;
        wp_reset_query();
     ?>
    </div>

<?php endif; ?>
